<?php
/**
 * This class is intended to run as a manager of the database class and provide an
 * easy and fast interface for the most common sql routines.
 *
 *
 * @author Nicanor Gioffré
 * @copyright 2010 GrayscaleInternational
 * @version 1.0
 * @package AgoraEngine
 * @subpackage AECore.DataHandler.forms
 * 
 */
class sqlInterface {
   // Variable que contiene los valores de POST o GET
   private $realVal;
   // Variables que controlan la visibilidad de los componentes visuales
   public $requireSelectorUI;
   public $requireEditorUI;
   // Contiene los errores provocados por los form objects
   public $rpsError;

   // Se establece que si no se establecio un submitter, se crea uno automaticamente
   public $includeStandardSubmiter = true;
   
   public function __construct ($values){
    // Establezco que los valores "input" van a provenir de $values
    $this->realVal = $values;
   }

   /*
    * Genera una nueva interfaz para el manejo automatico de datos sobre una tabla
    *  especifica.
    */
   public function newInterface($formObj,$dbAlias,$tblAlias,$dbVals,$formVals){

    /*
     * Etapa 1: Recolección y armado de datos
     */

    global $dbData;
    $tableUID = $dbData[$tblAlias]["uid"];
    //ARRANCA LA BASE DE DATOS
    $cnd = new database($dbAlias);
    $cnd->connect();

    // Obtengo el url de la pagina actual.
    $path = basename($_SERVER["REQUEST_URI"]);
    if (strpos($path,'?') == true) {
         // Si tiene parámetros adicionales, se los saco.
        $path = substr($path,0,strpos($path,'?'));
    }
    $this->neutralPath = $path;

    /*
     * Etapa 2: Analisis y Procesamiento de datos
     */

    // Si la operación a realizar es "crear" se genera el formulario.
    if($this->realVal["operation"] === "formCreate"){
        // Por defecto se crea un boton del tipo submit con el label indicado (Crear)
        if($this->includeStandardSubmiter){
            $formObj->newButton("sbmt","submit","Create",true,true);
        }
        
        // Se le indica al formulario cual es la 
        $formObj->action = "./".$path."?operation=formCreate&procedure=execute";
        // Lo
        $fpr = $formObj->formProcess($formVals);
        if($this->realVal["procedure"] === "execute"){
            foreach($formVals as $dbv){
                $fv[] = $this->realVal[$dbv];
            }
            if(is_array($fpr)){
                $this->rpsError = $fpr;
                $this->requireEditorUI = true;
                
                
            }else{
                $cnd->createQuickRegister($tblAlias,$dbVals,$fv);
            }
        }else{
            $this->requireEditorUI = true;
        }
    }


    //STAGE 3
    // Ejecución visual de la operación: Editar
    if($this->realVal["operation"] === "formEdit"){
        if($this->includeStandardSubmiter){
            $formObj->newButton("sbmt","submit","Edit",true,true);
        }
        //$formObj->field["holamundoform"]->action = "laputaquetepario.php";
       $cnd = new database($dbAlias);
       //$query = $cnd->search("engineLanguages","caption",array("lang","lang","lang"),"OR",array("fr","en","it"),array(array("uid","textid"),"ASC"),1,array(true),"primarydatabase");
       $formObj->action = "./".$path."?operation=formEdit&procedure=execute&id=".$this->realVal["id"];
       /*
        * Procedure Execute
        * 
        */
       if($this->realVal["procedure"] === "execute"){
            foreach($formVals as $dbv){
                $fv[] = $this->realVal[$dbv];
            }
            $cnd->editQuickRegister($tblAlias,$fv,$dbVals, $this->realVal["id"]);
        }
        /*
         * Procedure Selector
         *
         */

        if($this->realVal["procedure"] === "selector"){
             $this->requireSelectorUI = true;
             $this->dataContainerPushLink("Edit",$path."?operation=formEdit&procedure=editor","Edit","id=".$tableUID);
             $this->newDataContainer($tblAlias,$dbAlias,$dbVals,$path."?operation=formEdit&procedure=selector",$this->realVal);
        }

        /*
         * Procedure Selector
         *
         */

        if($this->realVal["procedure"] === "editor"){
            $query = $cnd->search($tblAlias,$dbVals,$tableUID,null,$this->realVal["id"],null,1, $return);
            
            $i = 0;
            foreach($formVals as $fvs){
                if($formObj->field[$fvs]->type == "selectbox"){
                 $formObj->field[$fvs]->selected = $query[$dbVals[$i]];
                }
           
                if($formObj->field[$fvs]->type == "checkbox"){
                    if($formObj->field[$fvs]->value == $query[$dbVals[$i]]){
                        $formObj->field[$fvs]->checked = "checked='checked'";
                    }
                    $i++;

                    continue;
                }
                $formObj->field[$fvs]->value = $query[$dbVals[$i]];
    
                $i++;
      
            }
            $this->requireEditorUI = true;
        }
    } //FIN DE FORM EDIT
    //STAGE 4
    if($this->realVal["operation"] === "formDelete"){
       $cnd = new database($dbAlias);
        if($this->realVal["procedure"] === "execute"){
            $cnd->deleteQuickRegister($tblAlias, $this->realVal["idNum"]);
        }
        if($this->realVal["procedure"] === "selector"){
            //BUSCO LOS VALORES
            //falta explicar los parametross       
            $this->requireSelectorUI = true;
            $this->dataContainerPushLink("delete",$path."?operation=formDelete&procedure=execute","Delete 2","idNum=".$tableUID);
            $this->newDataContainer($tblAlias,$dbAlias,$dbVals,$path."?operation=formDelete");
        }
    }
   }


   /**
    *
    * @var <type>
    * 
    */

   protected $linkLabels = array();
   protected $linkHTTPredirect = array();
   protected $path = array();
   protected $currentLink = array();
   protected $neutralPath;
   public function dataContainerPushLink($alias,$path,$linkLabels,$linkHTTPredirect){
    $this->linkLabels[] = $linkLabels;
    $this->linkHTTPredirect[] = $linkHTTPredirect;
    $this->path[] = $path;
   }


   // El campo/s (columna/s) que se usará como parámetro para ordenar los resultados
   protected $ordBy;
   // Los campos que se visualizarán una vez completada la busqueda
   protected $limitFields;
   
   //protected $limitNeedle;

   protected $stringLabel;
   // El orden en el que se ordenan los resultados (Ascendente o descendente)
   protected $orderDir;
   protected $startRange;
   protected $rpp;




   //
   protected $nextLinkPath;
   protected $previousLinkPath;
   protected $displayData;
   protected $dataLink;
   protected $labelLink;

   protected $tableUID;
   public function newDataContainer($tblAlias,$connectionAlias,$dbVals){
    // DATOS INICIALES
    // Datos de las tablas de las bases de datos disponibles:
    global $dbData;
    $this->tableUID = $dbData[$tblAlias]["uid"];
    
    // Inicia una nueva conexión a la base de datos.
    $getDb = new database($connectionAlias);
    // $this->realVal debe contener por lo menos los datos mas básicos para poder ordenar los resultados
    
    if(isset($this->realVal)){
        // Transfiero los valores de las variables externas a las variables internas
        $this->ordBy = $this->realVal["orderBy"];
        $this->orderDir = $this->realVal["orderDir"];
        $this->startRange = $this->realVal["startRange"];
        $this->rpp = $this->realVal["rpp"];
        // Si no hay un rango minimo seteado para la cantidad de registros que se piden por consulta, se setea ese limite a 10.
        if(!isset($this->rpp)){
            $this->rpp = 10; 
        }
        // Se establece la dirección que tendrá la "next page" (Considerando los proximos registros en la consulta)
        $nextPage = $this->startRange + $this->rpp;
        // Se establece la dirección que tendrá la "previous page" (Considerando los previos registros en la consulta)
        $previousPage = $this->startRange - $this->rpp;
        // Si se llega al rango de los primeros registros (entre 0 y $this->rpp) no hay previous page.
        if($previousPage<0){
            $previousPage = 0;
        }
        // PREVIOUS MAX PREVIOUS MIN A MODIFICAR
    }else{
        /*
         * Si no se proveen los datos o los metodos de donde sacar los datos para limitar las consultas, esto puede generar consultas
         * lentas que comprometan la estabilidad del sistema, por lo tanto en modo debug el sistema informa esta anomalidad.
         */
        panic2("DataHandler",1501,"<b>System was not provided with required external data nor specific method to obtain it.</b>",false);
    }
    // Se generan las variables que controlan los limites de busqueda en la consulta.
    if(isset($this->startRange) and isset($this->rpp)){
        $gLimit = array($this->startRange,$this->rpp);
    }else{
        $gLimit = $this->rpp;
    }
    if(!in_array($availableColumns,$dbVals)){
        $pusher = $dbVals;
        $pusher[] = $this->tableUID;
    }
    // EJECUCIÓN DE LA CONSULTA
    $data = $getDb->search($tblAlias,$pusher,$this->limitFields,null,$this->limitNeedle,array($this->ordBy,$this->orderDir),$gLimit,null,$dbAlias);

    // Disposición y arreglo de los datos
    $sc = 0;
    while($nsr = $getDb->db_fetch_search($data)){
        $cntr = -1;
        // Si $linkLabels es un string entonces hay solo un link asignado a cada valor
        //MODIFICAR A IS_STRING
        
        if(isset($this->linkLabels)){
            /* El sistema reconoce si se le pide el valor de una columna cuando se
             * le pasa como parámetro el nombre de una columna existente en la tabla
             * en la que se está operando. Si sólo se indica un valor el sistema busca
             * si ese valor es una columna de la base de datos en tal caso el resultado será:
             * "nombreDeColumna=$ValorDeEsaColumna  (Siendo $ValorDeEsaColumna el valor en la base
             * de datos para el registro particular que se esta analizando.
             */
                
             // Extraigo los posibles valores de columnas existentes en la base de datos
             $availableColumns = $getDb->retrieveColumns($tblAlias);
             foreach($this->linkLabels as $lL){    
                 $cntr++;
                 // Limpio la variable
                 $linkExt[$cntr] = "";
                 // Si $this->linkHTTPredirect es un array, entonces hay varios valores que se quieren pasar como parámetros al hipervinculo de cada registro
                 if(is_array($this->linkHTTPredirect[$cntr])){
                     //Recorro el $this->linkHTTPredirect para realizar el análisis con cada parámetro
                     foreach($this->linkHTTPredirect[$cntr] as $ile){        
                        //Detecto si el parámetro analizar tiene nombres de variables asignados
                        if (strpos($ile,'=') == true) {
                         // Extraigo el valor asignado al parámetro, que esta después del "=".
                         $staticCheck = substr($ile,strpos($ile,'=')+1,strlen($ile));
                         // Checkeo si ese parámetro existe en la tabla solicitada como nombre de columna
                         if(in_array($staticCheck,$availableColumns)){
                             // Genero un link con el nombre de variable solicitado y el valor extraido de la base de datos
                             $linkExt[$cntr] .= "&".substr($ile,0,strpos($ile,'='))."=".$nsr[$staticCheck];
                         }else{
                             // Genero un link con los datos estáticos propuestos.
                             $linkExt[$cntr] .= "&".$ile;
                         }       
                        }else{
                            // Si el parámetro que se esta analizándo no tiene "=" entonces:
                            if(in_array($ile,$availableColumns)){
                                // Si es el nombre de una columna de la tabla en la que se esta operando se crea
                                // un link del tipo valor=$val donde $val es el vaolor de la columna pedida en el
                                // registro correspondiente.
                                $linkExt[$cntr] .= "&".$ile."=".$nsr[$ile];
                            }
                        }
                    }
                }
                /* Si $this->linkHTTPredirect es un string se analiza únicamente ese valor ingresado
                * El proceso es identico al realizado anteriormente con la diferencia que
                * en este caso se analiza sólo un valor.
                */
                if(is_string($this->linkHTTPredirect[$cntr])){
                    if (strpos($this->linkHTTPredirect[$cntr],'=') == true) {
                        $staticCheck = substr($this->linkHTTPredirect[$cntr],strpos($this->linkHTTPredirect[$cntr],'=')+1,strlen($this->linkHTTPredirect[$cntr]));
                        if(in_array($staticCheck,$availableColumns)){
                            $linkExt[$cntr] = "&".substr($this->linkHTTPredirect[$cntr],0,strpos($this->linkHTTPredirect[$cntr],'='))."=".$nsr[$staticCheck];
                        }else{
                            $linkExt[$cntr] = "&".$this->linkHTTPredirect[$cntr];
                        }
                    }else{
                        if(in_array($this->linkHTTPredirect[$cntr],$availableColumns)){
                            $linkExt[$cntr] .= "&".$this->linkHTTPredirect[$cntr]."=".$nsr[$this->linkHTTPredirect[$cntr]];
                        }               
                    }
                }
            }    
        }  
        // DISPLAY / ALMACENAMIENTO DE LOS DATOS
        $this->displayData[] = $nsr;
        if(is_array($this->linkLabels)){
            // $c RECORRE LA CANTIDAD DE LABELS QUE HAY
            $c = 0;
            foreach($this->linkLabels as $links){
                // Si el path propuesto no tiene ? se le debe agregar.
                $questionMark = strpos($this->path[$c],"?") ? "" : "?";
                $this->dataLink[$sc][$c] = $this->path[$c].$questionMark.$linkExt[$c];
                $this->labelLink[$sc][$c] = $links;
                $c++;
            }
        }
        $sc++;
    }
    
    if(isset($this->ordBy)){
        (string) $this->ordBy = "orderBy=".$this->ordBy;
    }
    //$path aca es getCurrentPath();
    $nlMark = strpos($this->neutralPath[$c],"?") ? "" : "?";
    $this->nextLinkPath = $this->neutralPath.$nlMark."&startRange=".$nextPage."&rpp=".$this->rpp."&".$this->ordBy."&operation=".$this->realVal["operation"]."&procedure=".$this->realVal["procedure"];
    $this->previousLinkPath = $this->neutralPath.$nlMark."&startRange=".$previousPage."&rpp=".$this->rpp."&".$this->ordBy."&operation=".$this->realVal["operation"]."&procedure=".$this->realVal["procedure"];
       
   }

   public function getNextLink(){
       return $this->nextLinkPath;
   }
   
   public function getPreviousLink(){
       return $this->previousLinkPath;
   }
   private $gdTemp = 0;

   public function getData($excuseUnpushedData = true){ 
    
     if($excuseUnpushedData){
        //Saco los identificadores no visuales
        unset($this->displayData[$this->gdTemp][$this->tableUID]);
       }
        if(!isset($this->displayData[$this->gdTemp])){
            unset($return);
        }else{
            $return = $this->displayData[$this->gdTemp];
        }
     
       $this->gdTemp++;
       if(isset($return)){
        return $return;
       }else{
        $this->gdTemp = 0;
        return false;
       }
   }

   /** getDataLinkInGroups()
    * Al ejecutar la función devuelve el link/s de los registros analizados con
    * la función newDataContainer. Se devuelven de a uno por vez y se mueve el
    * cabezal hasta que se acaban los registros.
    *
    * @access public
    * @
    */
   private $tempRet = 0;
   public function getDataLinkInGroups(){
      for($i=0;$i<count($this->dataLink);$i++){
           if(!isset($this->dataLink[$this->tempRet])){
               unset($return);
           }else{
            $return["label"] = $this->labelLink[$this->tempRet];
            $return["data"] = $this->dataLink[$this->tempRet];
           }  
       }

       if(!count($return)<= 1){
           //$return = $return[0];
       }
       $this->tempRet++;
       if(isset($return)){
        return $return;
       }else{
        $this->tempRet = 0;
        return false;
        //return $return;
       }
   }
} // Fin de la clase
?>
