<?php
attachModule(STR, 'SearchFunctions');

class AECodeRunner {
    /**
     * Este array referencia al arreglo de datos con las variables globales de cada widget.
     * @var array()
     */
    private $GLOBAL_VARIABLES = array();

    /**
     * Este array contiene las variables internas de ejecucion.
     * @var array()
     */
    private $INT_VARIABLES = array();

    /**
     * Esta variable determina si se deben seguir ejecutando instrucciones.
     * @var boolean
     */
    private $EXECUTE = true;

    /**
     * Este array contiene la lista de todas las instrucciones soportadas por el AECodeRunner.
     * @var array()
     */
    private $AECODES = array(
            'REPLACE', // [ALL] Reemplaza texto dentro de un string.
            'WRITE', // Escribe un string al buffer
            'CONDEXEC', // Escribe un boolean al buffer
            'BUFFERFLUSH', // Vacia el buffer.
            'FORCEDUMP', // Detiene la ejecucion del script y devuelve el buffer
            'STOP', // Detiene la ejecucion del script
            'BUFFER', // Asigna el valor de buffer a la variable
            'LENGTH', // Asigna el largo de un string a una variable.
    );

    /**
     * Este array contiene la lista de todos los operandos internos soportados por el AECodeRunner.
     * @var array()
     */
    private $INTOPERANDS = array(
            '+', // SUMA
            '-', // RESTA
            '*', // MULTIPLICACION
            '/', // DIVISION
            '==', // EQUIVALE
            '>=', // MAYOR O IGUAL
            '<=', // MENOR O IGUAL
            '<>', // DISTINTO
            '>>', // MAYOR EXLUSIVO
            '<<', // MENOR EXCLUSIVO
            '!=', // NEGACION LOGICA
            '&&', // AND LOGICO
            '||', // OR LOGICO
    );

    /**
     * Este array contiene la lista de todos los operandos externos soportados por el AECodeRunner.
     * @var array()
     */
    private $EXTOPERANDS = array(
            '+=', // SUMA Y ASIGNACION
            '-=', // RESTA Y ASIGNACION
            '*=', // MULTIPLICACION Y ASIGNACION
            '/=', // DIVISION Y ASIGNACION
            '=', // ASIGNACION
            '->', // CAMBIO DE NOMBRE DE VARIABLES
            '-X', // ELIMINACION DE VARIABLE
    );

    /**
     * Descripcion de AECodeRunner
     *
     * Esta clase realiza un parsing de un fragmento de texto, reconociento y ejecutando
     * las instrucciones indicadas en AECode, verificando también el resultado por la
     * existencia de mas instrucciones en AECode.
     *
     * NOTA: Anteponer el caracter @ al inicio del tag, provoca que se ignore la ejecución
     * del mismo.
     *
     * @param array $GLOBAL_VARIABLES Referencia a las variables globales del widget.
     * @author kdewald
     * @version 1.0.0
     *
     */
    public function  __construct(&$GLOBAL_VARIABLES) {
        $this->GLOBAL_VARIABLES = &$GLOBAL_VARIABLES;

    }
    /**
     * @param string $text Texto a ser parseado.
     */
    public function parseAECODE($text) {
        // Recorro el texto en busca del tag de inicio del aecode.
        // Declaro primero la variable offset, para acortar el tiempo en las búsquedas sucesivas
        $offset = 0;
        while(@strpos($text, '[#AECODE',$offset)!==false) {
            // Guardo el inicio en una variable para hacer el asunto más rápido.
            $inicio = strpos($text, '[#AECODE',$offset);
            // Busco el final del aecode.
            $fin = strpos($text, 'AECODE#]',$offset);
            // Declaro la variable auxiliar offset2
            $offset2 = $inicio + 1;
            // Verifico si entremedio se esta abriendo algun otro tag.
            while (@strpos($text, '[#AECODE',$offset2)!==false) {
                // Marco el inicio del nuevo tag.
                $inicio2 = strpos($text, '[#AECODE',$offset2);
                // Si $inicio2 es mayor a $fin, el segundo tag está afuera del primero.
                if($inicio2 > $fin) {
                    // Salgo de la ejecucion del while
                    break;
                }else {
                    // Como $inicio2 es menor a $fin, el segundo tag esta adentro, debo desplazar el tag final.
                    $fin = strpos($text, 'AECODE#]',$fin+1);
                    // Desplazo el offset2
                    $offset2 = $inicio2 + 1;
                }
            }

            // Si $fin es falso, no hay tag de cierre, ignoro el primer tag de apertura encontrado y sigo buscando.
            if($fin===false) {
                $offset = $inicio + 1;
                continue;
            }else {
                // Ajusto $fin para que englobe al resto del aecode.
                $fin = $fin + 8;
            }
            // Verifico que el caracter en la posición anterior a inicio no sea un el caracter de escape.
            if(@mb_substr($text,($inicio-1), 1)=='@') {
                // Si es un @, avanzo a la próxima ejecución del ciclo while
                continue;
            }
            // Si pese a todo, la ejecución continua, extraigo el AECODE.
            $AECODE = substr($text,$inicio,$fin - $inicio);
            // Limpio el AECODE de los tags de apertura y cierre:
            // Marco el inicio del codigo a ejecutar.
            $inicioc = strpos($AECODE, '|') + 1;
            // Extraigo el aecode de los tags de apertura y cierre.
            $AECODE = substr($AECODE, $inicioc, strrpos($AECODE, '|') - $inicioc);
            // Limpio el aecode de espacios en blanco al inicio y al final.
            $AECODE = trim($AECODE);
            // Ejecuto el AECODE y lo asigno a la variable AERETURN.
            $AERETURN = $this->processAECODE($AECODE);
            // Reemplazo el AECODE por su código html correspondiente.
            $text = substr_replace($text, $AERETURN, $inicio,$fin - $inicio);
            // Genero el nuevo offset para ser usado en el ciclo siguiente, considerando el largo del nuevo string indicado.
            $offset = strlen($AERETURN) + $inicio;
        }
        // Como ya no quedan AECodes, devuelvo el texto procesado.
        return $text;
    }


    /**
     * @param string $code Tiene la forma: [#AECODE| instruccion1;instruccion2;instruccion3|AECODE#]
     */
    public function processAECODE($code) {
        // Instancio el buffer de retorno.
        $buffer = null;
        // Genero el array que contiene las instrucciones;
        $Instructions = array();
        // Extraigo la primera instruccion
        $instr = strtok($code, ';');
        // Mientras $instr no sea false, sigo agregando instrucciones a la lista.
        while ($instr !== false) {
            // Verifico primero si el caracter ; estaba escapado.
            // Si lo estaba, uno los dos fragmentos de codigo.
            while(EndsWith($instr, '@')) {
                $instr = $instr.';'.strtok(';');
            }
            $instr = trim($instr);
            $Instructions[] = $instr;
            $instr = strtok(';');
            /*
             * Necesito verificar aca que se estén agrupando correctamente los codigos
             * para las instrucciones if y while
            */
        }
        foreach ($Instructions as $instr) {
            $this->executeAECODEinstruction($instr,$buffer);
            if(!$this->EXECUTE) {
                break;
            }
        }
        if ($buffer != '')
            return $buffer;

    }

    /**
     * @param string $instruction Contiene la instruccion a ejecutarse.
     */
    public function executeAECODEinstruction($instruction,&$buffer) {
        // Extraigo el primer operando
        $operand = strtok($instruction, ' ');
        // Verifico si el operando esta en la lista de instrucciones soportadas
        if (in_array($operand, $this->AECODES)) {
            // Si el operando esta en la lista de instrucciones soportadas, se transfiere
            // control al operador de esa funcion.
            switch ($operand) {
                case 'REPLACE':
                // REPLACE data IN %Var WITH replacement
                // Extraigo el valor a reemplazar.
                    $value = $this->processCodes(trim(strtok(' ')));
                    // Extraigo el IN del texto.
                    $in = trim(strtok(' '));
                    // Extraigo la variable del texto.
                    $in = trim(strtok(' '));
                    if(StartsWith($in, '%%')) {
                        // Le quito los %%
                        $in = str_replace('%%', '', $in);
                        // Verifico si la variable existe, y si no, la creo.
                        if(!array_key_exists($in, $this->INT_VARIABLES)) {
                            $this->INT_VARIABLES[$in] = '';
                        }
                        $vararray =& $this->INT_VARIABLES;
                    }elseif(StartsWith($in, '%')) {
                        // Si hay %, creo la variable externa.
                        $in = str_replace('%', '', $in);
                        // Verifico si la variable existe, y si no, la creo.
                        if(!array_key_exists($in, $this->GLOBAL_VARIABLES)) {
                            $this->GLOBAL_VARIABLES[$in] = '';
                        }
                        $vararray =& $this->GLOBAL_VARIABLES;
                    }else {
                        // Si no hay variable especificada, error.
                        panic2('AECodeRunner', 8002, 'This instruction could not be completed: '.$instruction.' - Internal Memory Dump: '.print_r($this->INT_VARIABLES,true).' - Global Memory Dump: '.print_r($this->GLOBAL_VARIABLES,true), true);
                    }
                    // Extraigo el WITH del texto.
                    $with = trim(strtok(' '));
                    // Extraigo la variable del texto.
                    $with = strtok(' ');
                    $add = strtok(' ');
                    while ($add !== false) {
                        $with = $with.' '.$add;
                        $add = strtok(' ');
                    }
                    $vararray[$in] = str_replace($value, $with, $vararray[$in]);
                    break;
                case 'WRITE':
                // WRITE [Content]
                // Esta funcion escribe contenido al buffer, concatenandolo.
                    if (!is_string($buffer)) {
                        // Si el buffer no es un string, lo limpio.
                        $buffer = '';
                    }
                    $text = null;
                    $add = strtok(' ');
                    while ($add !== false) {
                        $text = $text.' '.$add;
                        $add = strtok(' ');
                    }
                    if (StartsWith($text, '@')) {
                        $buffer = $buffer.substr($text,1);
                    }else {
                        $buffer = $buffer.$this->processCodes($text);
                    }

                    break;
                case 'CONDEXEC':
                // Esta funcion evalúa una expresion booleana y la guarda en el buffer.
                    $buffer = ''; // Limpio primero el buffer.
                    $eval = $this->processCodes(trim(strtok(' '))); // Extraigo la expresion y la evaluo.
                    if ($eval) {
                        $buffer = 1;
                    }else {
                        $buffer = 0;
                    }
                    break;
                case 'BUFFERFLUSH':
                    $buffer = '';
                    break;
                case 'FORCEDUMP':
                    $this->EXECUTE=false;
                    break;
                case 'STOP':
                // Me fijo si hay mas variables
                    $text;
                    $add = strtok(' ');
                    while ($add !== false) {
                        $text = $text.' '.$add;
                        $add = strtok(' ');
                    }
                    // Si hay variables adicionales, las coloco en el buffer, sino, lo limpio.
                    if ($text != '') {
                        $buffer = $text;
                    }else {
                        $buffer = '';
                    }
                    $this->EXECUTE=false;
                    break;
                case 'BUFFER':
                // Guardo en una variable lo que contiene el buffer.
                    $in = trim(strtok(' '));
                    if(StartsWith($in, '%%')) {
                        // Le quito los %%
                        $in = str_replace('%%', '', $in);
                        // Verifico si la variable existe, y si no, la creo.
                        if(!array_key_exists($in, $this->INT_VARIABLES)) {
                            $this->INT_VARIABLES[$in] = '';
                        }
                        $vararray =& $this->INT_VARIABLES;
                    }elseif(StartsWith($in, '%')) {
                        // Si hay %, creo la variable externa.
                        $in = str_replace('%', '', $in);
                        // Verifico si la variable existe, y si no, la creo.
                        if(!array_key_exists($in, $this->GLOBAL_VARIABLES)) {
                            $this->GLOBAL_VARIABLES[$in] = '';
                        }
                        $vararray =& $this->GLOBAL_VARIABLES;
                    }else {
                        // Si no hay variable especificada, error.
                        panic2('AECodeRunner', 8002, 'This instruction could not be completed: '.$instruction.' - Internal Memory Dump: '.print_r($this->INT_VARIABLES,true).' - Global Memory Dump: '.print_r($this->GLOBAL_VARIABLES,true), true);
                    }
                    $vararray[$in] = $buffer;
                    break;
                case '':
                    break;
                default:
                    break;
            }

//                private $AECODES = array(
//            'REPLACE', // [ALL] Reemplaza texto dentro de un string.
//            'WRITE', // Escribe un string al buffer
//            'CONDEXEC', // Escribe un boolean al buffer
//            'BUFFERFLUSH', // Vacia el buffer.
//            'FORCEDUMP', // Detiene la ejecucion del script y devuelve el buffer
//            'STOP', // Detiene la ejecucion del script
//            'BUFFER', // Asigna el valor de buffer a la variable
//            'LENGTH', // Asigna el largo de un string a una variable.
//    );

        }else {
            // Si estoy aca, debo instanciar una variable.
            $variable = null;
            // Instancio la variable que apunta al arreglo de datos que estoy usando
            $vararray = array();
            // Quito espacios en blanco excedentes
            $operand = trim($operand);
            // Si hay %%, creo la variable interna.
            if(StartsWith($operand, '%%')) {
                // Le quito los %%
                $operand = str_replace('%%', '', $operand);
                // Verifico si la variable existe, y si no, la creo.
                if(!array_key_exists($operand, $this->INT_VARIABLES)) {
                    $this->INT_VARIABLES[$operand] = '';
                }
                $vararray =& $this->INT_VARIABLES;

            }elseif(StartsWith($operand, '%')) {
                // Si hay %, creo la variable externa.
                $operand = str_replace('%', '', $operand);
                // Verifico si la variable existe, y si no, la creo.
                if(!array_key_exists($operand, $this->GLOBAL_VARIABLES)) {
                    $this->GLOBAL_VARIABLES[$operand] = '';
                }
                $vararray =& $this->GLOBAL_VARIABLES;
            }else {
                // Si no hay %, perdono la falta y creo la variable interna.
                // Verifico si la variable existe, y si no, la creo.
                if(!array_key_exists($operand, $this->INT_VARIABLES)) {
                    $this->INT_VARIABLES[$operand] = '';
                }
                $vararray =& $this->INT_VARIABLES;
            }

            // Guardo el nombre de la variable
            $variable = $operand;
            // Extraigo el segundo operando.
            $operand = strtok(' ');
            // Quito espacios en blanco excedentes
            $operand = trim($operand);
            // Reviso si es un operando soportado
            switch ($operand) {
                case '=':
                // Operador de asignacion
                // Como estoy usando el operador de asignacion, asigno el valor a la variable
                    $vararray[$variable] = $this->processCodes(trim(strtok(' ')));
                    break;
                case '+=':
                // Operador de suma y asignacion
                    $vararray[$variable] = $vararray[$variable] + $this->processCodes(trim(strtok(' ')));
                    break;
                case '-=':
                // Operador de resta y asignacion
                    $vararray[$variable] = $vararray[$variable] - $this->processCodes(trim(strtok(' ')));
                    break;
                case '*=':
                // Operador de multiplicacion y asignacion
                    $vararray[$variable] = $vararray[$variable] * $this->processCodes(trim(strtok(' ')));
                    break;
                case '/=':
                // Operador de division y asignacion
                    $divisor = $this->processCodes(trim(strtok(' ')));
                    if (!$divisor)
                        $vararray[$variable] = $vararray[$variable] + $divisor;
                    break;
                case '->':
                // Operador de cambio de nombre de variable.
                // Extraigo el nombre de la nueva variable
                    $newvar = trim(strtok(' '));
                    // Ejecuto AECODE para asignarle el valor de la variable actual.
                    $cod = $newvar.' = '.$vararray[$variable];
                    $this->executeAECODEinstruction($cod, $buffer);
                    // Elimino la variable vieja.
                    unset ($vararray[$variable]);
                    break;
                case ('-X' || '-x'):
                // Operador de eliminacion de variable
                    unset ($vararray[$variable]);
                    break;
                case '':
                    break;
                default:
                // Genero un reporte del error

                // Devuelvo false para indicar que hubo error.
                    break;
            }
        }
    }


    private function processCodes($operation) {
        // Esta función procesa operaciones aritmétricas simples y extrae variables

        //NOTA: Es necesario mejorar esto para que ejecute operaciones compuestas con parentesis

        // Verifico si la operación esta escapada.
        if(StartsWith($operation, '@')) {
            // Si la operacion esta escapada, devuelvo la variable sin el @
            return substr($operation,1);
        }else {
            // Hago una busqueda de operandos en la operación
            $operands = srcharraylist($operation, $this->INTOPERANDS);
            // Verifico el resultado del operando.
            if($operands === false) {
                // Si no hay operando, estoy llamando a una variable.
                if(StartsWith($operation, '%%')) {
                    // Le quito los %%
                    $operation = str_replace('%%', '', $operation);
                    // Verifico si la variable existe, y si no, la creo.
                    if(!array_key_exists($operation, $this->INT_VARIABLES)) {
                        $this->INT_VARIABLES[$operation] = '';
                    }
                    return $this->INT_VARIABLES[$operation];
                }elseif(StartsWith($operation, '%')) {
                    // Si hay %, creo la variable externa.
                    $operation = str_replace('%', '', $operation);
                    // Verifico si la variable existe, y si no, la creo.
                    if(!array_key_exists($operation, $this->GLOBAL_VARIABLES)) {
                        $this->GLOBAL_VARIABLES[$operation] = '';
                    }
                    return $this->GLOBAL_VARIABLES[$operation];
                }else {
                    return $operation;
                }
            }else {
                // Si hay al menos un operando, tengo que extraer ambos terminos.
                $term1 = strstr($operation, $operands,true);
                // Me fijo si tengo que procesarlo, o es una variable.
                if(srcharraylist($term1, $this->INTOPERANDS) === false) {
                    // Si no hay operando, estoy llamando a una variable.
                    if(StartsWith($term1, '%%')) {
                        // Le quito los %%
                        $term1 = str_replace('%%', '', $term1);
                        // Verifico si la variable existe, y si no, la creo.
                        if(!array_key_exists($term1, $this->INT_VARIABLES)) {
                            $this->INT_VARIABLES[$term1] = '';
                        }
                        $term1 = $this->INT_VARIABLES[$term1];
                    }elseif(StartsWith($term1, '%')) {
                        // Si hay %, creo la variable externa.
                        $term1 = str_replace('%', '', $term1);
                        // Verifico si la variable existe, y si no, la creo.
                        if(!array_key_exists($term1, $this->GLOBAL_VARIABLES)) {
                            $this->GLOBAL_VARIABLES[$term1] = '';
                        }
                        $term1 = $this->GLOBAL_VARIABLES[$term1];
                    }
                }
                else {
                    // Si tiene operandos, los ejecuto.
                    $term1 = $this->processCodes($term1);
                }

                // Extraigo el segundo termino.
                $term2 = substr(strstr($operation, $operands),1);
                // Me fijo si tengo que procesarlo, o es una variable.
                if(srcharraylist($term2, $this->INTOPERANDS) === false) {
                    // Si no hay operando, estoy llamando a una variable.
                    if(StartsWith($term2, '%%')) {
                        // Le quito los %%
                        $term2 = str_replace('%%', '', $term2);
                        // Verifico si la variable existe, y si no, la creo.
                        if(!array_key_exists($term2, $this->INT_VARIABLES)) {
                            $this->INT_VARIABLES[$term2] = '';
                        }
                        $term2 = $this->INT_VARIABLES[$term2];
                    }elseif(StartsWith($term2, '%')) {
                        // Si hay %, creo la variable externa.
                        $term2 = str_replace('%', '', $term2);
                        // Verifico si la variable existe, y si no, la creo.
                        if(!array_key_exists($term2, $this->GLOBAL_VARIABLES)) {
                            $this->GLOBAL_VARIABLES[$term2] = '';
                        }
                        $term2 = $this->GLOBAL_VARIABLES[$term2];
                    }
                }
                else {
                    // Si tiene operandos, los ejecuto.
                    $term2 = $this->processCodes($term2);
                }


                // Realizo un switch para saber el tipo de operación
                switch ($operands) {
                    case '+':
                    // Operacion Suma
                    // Si alguno de los terminos es nulo, lo convierto en un 0.
                        if($term1==null) {
                            $term1 = 0;
                        }
                        if($term2==null) {
                            $term2 = 0;
                        }
                        if(is_numeric($term1)&&is_numeric($term2)) {
                            // Si son numeros, sumo
                            return $term1 + $term2;
                        }else {
                            return $term1.$term2;
                        }
                        break;
                    case '-':
                    // Operacion Resta
                    // Si alguno de los terminos es nulo, lo convierto en un 0.
                        if($term1==null) {
                            $term1 = 0;
                        }
                        if($term2==null) {
                            $term2 = 0;
                        }
                        if(is_numeric($term1)&&is_numeric($term2)) {
                            // Si son numeros, los resto
                            return $term1 - $term2;
                        }else {
                            // Si no son numeros, genero un error crítico.
                            panic2('AECodeRunner', 8001, 'This operation could not be completed: '.$operation.' - Internal Memory Dump: '.print_r($this->INT_VARIABLES,true).' - Global Memory Dump: '.print_r($this->GLOBAL_VARIABLES,true), true);


                        }
                        break;
                    case '*':
                    // Operacion Multiplicacion
                    // Si alguno de los terminos es nulo, lo convierto en un 0.
                        if($term1==null) {
                            $term1 = 0;
                        }
                        if($term2==null) {
                            $term2 = 0;
                        }
                        // Realizo una comparacion para averiguar si el numero es un integer.
                        // Si la comparacion es true, hago un cast a integer.
                        if($term2 === (string)(int)$term2) {
                            $term2 = (int)$term2;
                        }
                        if ($term1 === (string)(int)$term1) {
                            $term1 = (int)$term1;
                        }
                        if(is_numeric($term1)&&is_numeric($term2)) {
                            // Si son ambos numeros, los multiplico
                            return $term1 * $term2;
                        }elseif(is_string($term1)&&is_int($term2)) {
                            // Si uno es un string y el otro un integer, los concateno la cantidad de veces indicada.
                            $aux = $term1;
                            $i = $term2;
                            while($i>1) {
                                $term1 = $term1.$aux;
                                $i--;
                            }
                            return $term1;
                        }elseif(is_string($term2)&&is_int($term1)) {
                            // Si uno es un string y el otro un integer, los concateno la cantidad de veces indicada.
                            $aux = $term2;
                            $i = $term1;
                            while($i>1) {
                                $term2 = $term2.$aux;
                                $i--;
                            }
                            return $term2;
                        }else {
                            // Si no son numeros, genero un error crítico.
                            panic2('AECodeRunner', 8001, 'This operation could not be completed: '.$operation.' - Internal Memory Dump: '.print_r($this->INT_VARIABLES,true).' - Global Memory Dump: '.print_r($this->GLOBAL_VARIABLES,true), true);
                        }
                        break;
                    case '/':
                    // Operacion Division
                    // Si el divisor es nulo, lo convierto en un 1, para evitar problemas de division.
                        if($term2==null || $term2 == '0' || $term2 == 0) {
                            $term2 = 1;
                        }
                        if(is_numeric($term1)&&is_numeric($term2)) {
                            // Si son numeros, los divido
                            return $term1 / $term2;
                        }else {
                            // Si no son numeros, genero un error crítico.
                            panic2('AECodeRunner', 8001, 'This operation could not be completed: '.$operation.' - Internal Memory Dump: '.print_r($this->INT_VARIABLES,true).' - Global Memory Dump: '.print_r($this->GLOBAL_VARIABLES,true), true);
                        }
                        break;
                    case '==':
                    // Operacion Equivalencia
                    // Si alguno es true o false en formato texto, lo convierto a boolean
                        if($term1=='true') {
                            $term1 = 1;
                        }
                        if($term1=='false') {
                            $term1 = 0;
                        }
                        if($term2=='true') {
                            $term2 = 1;
                        }
                        if($term2=='false') {
                            $term2 = 0;
                        }
                        if($term1 == $term2) {
                            // True = 1
                            return 1;
                        }else {
                            return 0;
                        }
                        break;
                    case '>=':
                    // Operacion Mayor o Igual
                    // Si alguno de los terminos es nulo, lo convierto en un 0.
                        if($term1==null) {
                            $term1 = 0;
                        }
                        if($term2==null) {
                            $term2 = 0;
                        }
                        if(is_numeric($term1)&&is_numeric($term2)) {
                        }elseif(is_string($term1)&&is_int($term2)) {
                            // Si uno es un string y el otro un integer, utilizo el largo del string para comparar.
                            $term1 = strlen($term1);
                        }elseif(is_string($term2)&&is_int($term1)) {
                            // Si uno es un string y el otro un integer, utilizo el largo del string para comparar.
                            $term2 = strlen($term2);
                        }else {
                            // Como ambos son strings, considero el largo de ambos
                            $term1 = strlen($term1);
                            $term2 = strlen($term2);
                        }
                        If($term1 >= $term2) {
                            return 1;
                        }else {
                            return 0;
                        }
                        break;
                    case '<=':
                    // Operacion Menor o Igual
                    // Si alguno de los terminos es nulo, lo convierto en un 0.
                        if($term1==null) {
                            $term1 = 0;
                        }
                        if($term2==null) {
                            $term2 = 0;
                        }
                        if(is_numeric($term1)&&is_numeric($term2)) {
                        }elseif(is_string($term1)&&is_int($term2)) {
                            // Si uno es un string y el otro un integer, utilizo el largo del string para comparar.
                            $term1 = strlen($term1);
                        }elseif(is_string($term2)&&is_int($term1)) {
                            // Si uno es un string y el otro un integer, utilizo el largo del string para comparar.
                            $term2 = strlen($term2);
                        }else {
                            // Como ambos son strings, considero el largo de ambos
                            $term1 = strlen($term1);
                            $term2 = strlen($term2);
                        }
                        If($term1 <= $term2) {
                            return 1;
                        }else {
                            return 0;
                        }
                        break;
                    case ('<>' || '!='):
                    // Operacion Diferencia
                    // Si alguno es true o false en formato texto, lo convierto a boolean
                        if($term1=='true') {
                            $term1 = 1;
                        }
                        if($term1=='false') {
                            $term1 = 0;
                        }
                        if($term2=='true') {
                            $term2 = 1;
                        }
                        if($term2=='false') {
                            $term2 = 0;
                        }
                        if($term1 != $term2) {
                            // True = 1
                            return 1;
                        }else {
                            return 0;
                        }
                        break;
                    case '>>':
                    // Operacion Mayor estricto
                    // Si alguno de los terminos es nulo, lo convierto en un 0.
                        if($term1==null) {
                            $term1 = 0;
                        }
                        if($term2==null) {
                            $term2 = 0;
                        }
                        if(is_numeric($term1)&&is_numeric($term2)) {
                        }elseif(is_string($term1)&&is_int($term2)) {
                            // Si uno es un string y el otro un integer, utilizo el largo del string para comparar.
                            $term1 = strlen($term1);
                        }elseif(is_string($term2)&&is_int($term1)) {
                            // Si uno es un string y el otro un integer, utilizo el largo del string para comparar.
                            $term2 = strlen($term2);
                        }else {
                            // Como ambos son strings, considero el largo de ambos
                            $term1 = strlen($term1);
                            $term2 = strlen($term2);
                        }
                        If($term1 > $term2) {
                            return 1;
                        }else {
                            return 0;
                        }
                        break;
                    case '<<':
                    // Operacion Menor estricto
                    // Si alguno de los terminos es nulo, lo convierto en un 0.
                        if($term1==null) {
                            $term1 = 0;
                        }
                        if($term2==null) {
                            $term2 = 0;
                        }
                        if(is_numeric($term1)&&is_numeric($term2)) {
                        }elseif(is_string($term1)&&is_int($term2)) {
                            // Si uno es un string y el otro un integer, utilizo el largo del string para comparar.
                            $term1 = strlen($term1);
                        }elseif(is_string($term2)&&is_int($term1)) {
                            // Si uno es un string y el otro un integer, utilizo el largo del string para comparar.
                            $term2 = strlen($term2);
                        }else {
                            // Como ambos son strings, considero el largo de ambos
                            $term1 = strlen($term1);
                            $term2 = strlen($term2);
                        }
                        If($term1 < $term2) {
                            return 1;
                        }else {
                            return 0;
                        }
                        break;
                    case '&&':
                    // Operacion AND
                    // Si alguno es true o false en formato texto, lo convierto a boolean
                        if($term1=='true') {
                            $term1 = 1;
                        }
                        if($term1=='false') {
                            $term1 = 0;
                        }
                        if($term2=='true') {
                            $term2 = 1;
                        }
                        if($term2=='false') {
                            $term2 = 0;
                        }
                        if (is_numeric($term1) && is_numeric($term2)) {
                            if($term1 && $term2) {
                                // True = 1
                                return 1;
                            }else {
                                return 0;
                            }
                        } else {
                            // Si no son numeros, genero un error crítico.
                            panic2('AECodeRunner', 8001, 'This operation could not be completed: '.$operation.' - Internal Memory Dump: '.print_r($this->INT_VARIABLES,true).' - Global Memory Dump: '.print_r($this->GLOBAL_VARIABLES,true), true);
                        }
                        break;
                    case '||':
                    // Operacion OR
                    // Si alguno es true o false en formato texto, lo convierto a boolean
                        if($term1=='true') {
                            $term1 = 1;
                        }
                        if($term1=='false') {
                            $term1 = 0;
                        }
                        if($term2=='true') {
                            $term2 = 1;
                        }
                        if($term2=='false') {
                            $term2 = 0;
                        }
                        if (is_numeric($term1) && is_numeric($term2)) {
                            if($term1 || $term2) {
                                // True = 1
                                return 1;
                            }else {
                                return 0;
                            }
                        } else {
                            // Si no son numeros, genero un error crítico.
                            panic2('AECodeRunner', 8001, 'This operation could not be completed: '.$operation.' - Internal Memory Dump: '.print_r($this->INT_VARIABLES,true).' - Global Memory Dump: '.print_r($this->GLOBAL_VARIABLES,true), true);
                        }
                        break;
                    default:
                    // Si no reconozco el operando, devuelvo el valor nulo.
                        panic2('AECodeRunner', 8000, 'This operation could not be completed: '.$operation);
                        return null;
                        break;
                }

            }
        }
    }
}
?>
