<?php
/* 
 * BBparser: Funciones que procesan el bbcode
*/

attachModule(STR, 'SearchFunctions');
attachModule(STR, 'WebFunctions');

function processBBcode($text) {
    // Esta función lee un texto y busca si contiene bbcode, y de ser así, lo ejecuta

    // Declaro variable offset
    $offset = 0;
    $status = 1;
    while ($status != 0) {
        // Busco si hay algún bbcode.
        $srch = srchbbcode($text, $offset);
        // Si me devolvieron un array, significa que se pudo extraer bbcode.
        // De lo contrario, proceso el codigo de error correspondiente.
        if (is_array($srch)) {
            // Al recibir un array, significa que se encontró un bbcode valido.
            // Array(bbcode, inicio)

            //Ejecuto el bbcode extraido y guardo en la variable bbtext el texto devuelto.
            $bbtext = executeBBcode($srch[0]);
            // Reemplazo el bbcode por su bbtext correspondiente.
            $text = substr_replace($text, $bbtext, $srch[1] ,strlen($srch[0]));
            // Desplazo el offset para seguir buscando en el resto del documento.
            $offset = $srch[1] + strlen($srch[0]);
            // Fijo el status en 1, para seguir buscando más bbcodes.
            $status = 1;
        }else {
            switch ($srch) {
                case 0:
                // Codigo de error 0: No hay bbcode
                // Indico que debo finalizar el loop poniendo status en 0.
                    $status = 0;
                    break;
                case 1:
                // Código de error 1: No hay tag de cierre.
                // Incremento en 2 unidades al offset, para desplazar el
                // inicio de la busqueda de mas bbcode, a fin que no quede
                // atrapado en un loop infinito analizando siempre el mismo.
                    $offset++;
                    $status = 1;
                    break;
            }
        }
    }
    return $text;


}
function executeBBcode($code) {
    // Esta función interpreta un bbcode, llamando a la función asociada y pasándole
    // todos los parámetros para que se pueda traducir al código html correspondiente.

    // Paso 1: Identifico el tag leyendo el de cierre.
    // $initag es el indice del caracter / en el tag de cierre + 1.
    $initag = strrpos($code,'/') + 1;
    // $taglen es el largo del tag.
    $taglen = strlen($code)- $initag - 1;
    // Una vez que tengo el inicio y el largo del tag, lo extraigo del code.
    $tag = substr($code, $initag, $taglen);

    // initag y taglen los uso ahora para extraer el texto entre los tags de apertura y cierre.
    $initag = strpos($code,']') + 1;
    $taglen = strrpos($code,'[') - $initag;
    $intcode = substr($code,$initag,$taglen);
    // Si el tag es code, implica que tengo que devolver el contenido de adentro sin llamar a ninguna funcion.
    // De lo contrario, reviso si adentro no quedó algún otro bbcode para procesar.
    if ($tag == 'code') {
        return $intcode;
    }else {
        // Proceso los bbcodes internos, si es que los hay.
        $intcode = processBBcode($intcode);
        // Para eso, reemplazo el código interno por su bbcode correspondiente.
        $code = substr_replace($code,$intcode,$initag,$taglen);
        // NOTA: Verificar que el replace se haga correctamente.
        switch ($tag) {
            //Busco el tag que corresponde y lo ejecuto.
            case 'b':
            // Devuelvo el texto escrito en bold.
                $code = '<span style="font-weight: bold;">'.$intcode.'</span>';
                break;
            case 'u':
            // Devuelvo el texto subrayado.
                $code = '<span style="text-decoration: underline;">'.$intcode.'</span>';
                break;
            case 'i':
            // Devuelvo el texto en itálica.
                $code = '<em>'.$intcode.'</em>';
                break;
            case 's':
            // Devuelvo el texto tachado.
                $code = '<del>'.$intcode.'</del>';
                break;
            case 'color':
            // Cambio el color del texto.
                $data = extractBBdata($code);
                $code = '<span style="color: '.$data.';">'.$intcode.'</span>';
                unset($data);
                break;
            case 'size':
            // Cambio el tamaño del texto
                $data = extractBBdata($code);
                $code = '<span style="font-size: '.$data.'px;">'.$intcode.'</span>';
                unset($data);
                break;
            case 'font':
            // Cambio la fuente del texto
                $data = extractBBdata($code);
                $code = '<span style="font-family: '.$data.';">'.$intcode.'</span>';
                unset($data);
                break;
            case 'left':
            // Alinea el texto a la izquierda
                $code = '<div style="text-align: '.$tag.';">'.$intcode.'</div>';
                break;
            case 'center':
            // Alinea el texto al centro
                $code = '<div style="text-align: '.$tag.';">'.$intcode.'</div>';
                break;
            case 'right':
            // Alinea el texto a la derecha
                $code = '<div style="text-align: '.$tag.';">'.$intcode.'</div>';
                break;
            case 'quote':
            // Cita el texto dentro.
                if(strpos($code, '['.$tag.'=')!== false) {
                    $data = extractBBdata($code);
                    $code = insertTag('br', true, false, true,null,true).
                            $data.
                            ' wrote: '.
                            insertTag('br', true, false, true,null,true).
                            insertTag('blockquote', true, false, false,null,true).
                            $intcode.
                            insertTag('blockquote', false, false, false,null,true);
                    unset($data);
                }else {
                    $code = insertTag('br', true, false, true,null,true).
                            'Quote: '.
                            insertTag('br', true, false, true,null,true).
                            insertTag('blockquote', true, false, false,null,true).
                            $intcode.
                            insertTag('blockquote', false, false, false,null,true);
                }
                break;
            case 'list':
            // Crea una lista.

            // Reviso primero si hay bulletpoints
                if(strpos($intcode,'[*]')===false) {
                    // Si no hay bulletpoints, devuelvo el intcode sin cambios
                    $code = $intcode;
                }else {
                    // Hago un trim del intcode para quitar espacios sobrantes.
                    $intcode = trim($intcode);
                    // Preparo un array para los items.
                    $Items = array();
                    //Extraigo el primer item y le quito los espacios.
                    $aux = strtok($intcode, '[*]');
                    $aux = trim($aux);
                    // Lo guardo en $Items y me fijo si hay mas.
                    $Items[] = $aux;
                    $cont = true;
                    do {
                        $aux = strtok('[*]');
                        if ($aux !== false) {
                            // Si aux no es false, agrego el item a la lista.
                            $aux = trim($aux);
                            $Items[] = $aux;
                        }else {
                            //Si $aux es false, significa que no hay mas items, seteo cont en false
                            $cont = false;
                        }
                    }while($cont);
                    // Me fijo si se dio alguna indicacion especial para los bulletpoints.
                    $data = extractBBdata($code);
                    if ($data === false) {
                        $code = '';
                        foreach ($Items as $value) {
                            $code = $code.insertTag('li',true,false,false,null,true).$value;
                        }
                    }elseif ($data == '1') {
                        $code = insertTag('ol',true,false,false,array('style'=>'list-style-type: decimal'),true);
                        foreach ($Items as $value) {
                            $code = $code.insertTag('li',true,false,false,null,true).$value;
                        }
                        $code = $code.insertTag('ol',false,false,false,null,true);
                    }elseif ($data == 'a') {
                        $code = insertTag('ol',true,false,false,array('style'=>'list-style-type: lower-alpha'),true);
                        foreach ($Items as $value) {
                            $code = $code.insertTag('li',true,false,false,null,true).$value;
                        }
                        $code = $code.insertTag('ol',false,false,false,null,true);
                    }

                }
                break;
            case 'url':
            // Crea un link. Si se pasa como parámetro del link un número, el linkwidget
            // sobreentiende que se trata de una página del sistema, lo mismo ocurre si
            // se indica un nombre de archivo que no sea una dirección web.
                if(strpos($code, '['.$tag.'=')!== false) {
                    $link = extractBBdata($code);
                    // Genero el link
                    $link = processHyperLink($link);
                    $code = insertTag('a', true, false, false,array('href'=>$link),true).
                            $intcode.
                            insertTag('a', false, false, false,null,true);
                    unset($link);
                }else {
                    // Como el link es externo, el texto está entre los tags.
                    // Genero el link
                    $code = insertTag('a', true, false, false,array('href'=>$intcode),true).
                            $intcode.
                            insertTag('a', false, false, false,null,true);
                }
                break;
            case 'email':
            // Crea un link para enviar un correo electrónico.
                if(strpos($code, '['.$tag.'=')!== false) {
                    $link = extractBBdata($code);
                    $link = 'mailto:'.$link.'';
                    // Genero el link
                    $code = insertTag('a', true, false, false,array('href'=>$link),true).
                            $intcode.
                            insertTag('a', false, false, false,null,true);
                    unset($link);
                }else {
                    // Como el link es externo, el texto está entre los tags.
                    // Genero el link
                    $intcode = 'mailto:'.$intcode.'';
                    $code = insertTag('a', true, false, false,array('href'=>$intcode),true).
                            $intcode.
                            insertTag('a', false, false, false,null,true);
                }
                break;
            case 'img':
            // Crea una imagen.

            // Si en AEData/Images se encuentra la imagen, la direcciono allí
                if(file_exists('./AEData/Images/'.$intcode)) {
                    //Si existe, fijo un path relativo
                    //CORRECCION PARA QUE FUNCIONE EN DIRECTORIO EXTERNO
                    $code = insertTag('img', true, false, true, array('src'=>'/agoraengine/AEData/Images/'.$intcode),true);
                }else {
                    //Si no existe, el path es absoluto.
                    $code = insertTag('img', true, false, true, array('src'=>$intcode),true);
                }
                break;
            case '':

                break;
            case '':

                break;
        }
    }
    return $code;
}
function extractBBdata($code) {
// Busco si hay algun = al comienzo.
    $ini = strpos($code, '=');
    if ($ini !== false) {
        // Si hay un =, extraigo el resto del texto.
        $ini = $ini+1;
        $fin = strpos($code, ']');
        if($fin<$ini) {
            return false;
        }else {
            return substr($code,$ini,$fin-$ini);
        }
    }else {
        // De no haber datos a extraer, devuelvo false.
        return false;
    }
}

?>
