<?php
/**
 * SearchFunctions
 *
 * Este paquete de funciones tiene la finalidad de realizar búsquedas avanzadas
 * ya sea en textos, como en bases de datos, otras páginas y demás.
 */

function srchpos($haystack,$needle,$wildcard=null) {
    /* Esta función permite buscar un determinado string dentro de un texto,
     * con la facilidad que si de indica un wildcard (o comodín), se reemplazará
     * al wildcard por cualquier string que se encuentre en el medio entre el
     * resto del texto indicado. El needle puede manejar todos los wildcards que
     * se deseen.
    */
    if (isset($wildcard)) {
        // Primero cuanto la cantidad de wildcards que tengo
        $amount = substr_count($needle,$wildcard);
        // Creo el array donde guardo lo que tengo que buscar en cada parte.
        $parts = array();
        // Cargo en el array el primer fragmento.
        $parts[] = strtok($needle, $wildcard);
        // Recorro el needle para cargar el resto de los fragmentos.
        for($i = 0; $i<$amount;$i++) {
            $parts[] = strtok($wildcard);
        }
        // Sumo uno a amount para que cuente la cantidad total de partes que tiene
        // el string que estoy buscando.
        $amount++;
        // Busco la primer parte del needle. Si no la encuentro, devuelvo false.
        // Si la encuentro, toca asegurarse que el resto del string esté presente.
        $return = strpos($haystack, $parts[0]);
        if ($return === false) {
            return false;
        }else {
            // Fijo un offset para acortar el proceso de busqueda.
            $offset = $return;
            // Busco si a continuación están presentes el resto de los elementos.
            for($i = 1; $i<$amount;$i++) {
                // Busco el elemento siguiente.
                $srch = strpos($haystack, $parts[$i], $offset);
                // Evaluo el resultado de la busqueda.
                if ($srch === false) {
                    // Si dio false, significa que todo el string indicado no
                    // esta presente, salgo del loop y devuelvo false.
                    $return = false;
                    break;
                }else {
                    // Si la busqueda dio true, desplazo el offset.
                    $offset = $srch;
                }
                // Paso al proximo elemento.
            }
            return $return;
        }
    }else {
        // Si no hay wildcard seteado, la posicion de inicio es aquella donde se
        // encuentra el needle.
        return strpos($haystack, $needle);
    }
}
function srchlen($haystack,$needle,$wildcard=null) {
    if (isset($wildcard)) {

    }else {
        // Si no hay wildcard seteado, el largo de la cadena que se busca es igual
        // al largo del needle.
        return count_chars($needle);
    }
}
function srchbbcode($haystack,$offset) {
    /**
     * Esta función esta pensada para buscar strings con la estructura de un bbcode,
     * y devolver un array conteniendo el string, la posición de inicio.
     */
    /* CODIGOS DE ERROR DEVUELTOS
     *
     * 0: No se encontro un tag de apertura completo.
     * 1: No se encontró tag de cierre para el primer tag de apertura encontrado.
    */

    //NOTA: La estructura de lo que se está buscando es [#]*[/#], donde * es
    // cualquier string, y ambos # tienen que ser el mismo string.

    //Primero busco el primer tag
    $inicio = @strpos($haystack, '[',$offset);
    // Si hay tag de apertura, sigo procesando, de lo contrario, devuelvo false.
    if($inicio !== false) {
        // Busco el tag de cierre.
        $fin = strpos($haystack, ']',$inicio);
        // Si hay tag de cierre, sigo procesando, sino devuelvo false.
        if($fin !== false) {
            // Cargo el openingtag con la finalidad de procesarlo despues.
            $openingtag = substr($haystack, $inicio, $fin-$inicio);
            // Me fijo si el openintag recibió algún parámetro.
            $param = strpos($openingtag,'=');
            if($param!==false) {
                // Si recibió algún parámetro, lo ignoro para extraer el tag inicial.
                $openingtag = substr($openingtag,1,$param-1);
            }else {
                // Si no recibió un parámetro, la cadena es el openingtag.
                $openingtag = substr($openingtag,1);
            }
            // Una vez aislado el openingtag, genero un closingtag y busco si existe
            $closingtag = '[/'.$openingtag.']';
            $fin = strpos($haystack, $closingtag);
            if ($fin!==false) {
                // Como hay tag de cierre, aislo el string completo y armo el array para devolverlo
                // Primero localizo el ultimo caracter del string.
                $fin = $fin +strlen($closingtag);
                // Recorto la porcion de texto que contiene el bbcode
                $bbcode = substr($haystack,$inicio,$fin-$inicio);
                // Creo el array y lo asigno a return para devolverlo
                $return = array($bbcode,$inicio);
            }else {
                // Si no hay tag de cierre, devuelvo el número 1
                $return = 1;
            }
        }else {
            // Asigno a return un 0
            $return = 0;
        }
    }else {
        // Asigno a return un 0
        $return = 0;
    }
    return $return;
}
function srcharraylist($haystack,$needles) {
    /*
     * Esta función busca en el haystack los needles del array, uno por uno.
     * Si encuentra alguno, devuelve el primero que encontro, de lo contrario, false.
    */
    if(is_array($needles)) {
        foreach($needles as $needle) {
            // Verifico que la needle no sea un objeto nulo
            if($needle != '') {
                if(strpos($haystack, $needle)!==false) {
                    return $needle;
                }
            }
        }
        return false;
    }else {
        return false;
    }
}
function StartsWith($haystack,$needle,$case=true) {
    if($case) {
        return strpos($haystack, $needle, 0) === 0;
    }else {
        return stripos($haystack, $needle, 0) === 0;
    }
}
function EndsWith($haystack,$needle,$case=true) {
    $expectedPosition = strlen($haystack) - strlen($needle);
    // NOTA: En la funcion original, a expectedposition se le restaba una unidad antes de compararlo.
    // Se lo saco porque devolvía un resultado erroneo.
    if($case) {
        return strrpos($haystack, $needle, 0) === $expectedPosition;
    }else {
        return strripos($haystack, $needle, 0) === $expectedPosition;
    }
}

?>
