<?php

/* En este archivo se agrupan una serie de funciones de uso común en varias clases. */

function insertTag($name, $opening, $newline, $finalclose, $AttribArray=null, $astext = false) {
    /* Esta función abre y cierra Tags siguiendo el formato HTML. En name se indica
     * el nombre del tag a abrirse, y en opening se indica si el tag se está abriendo o
     * cerrando. Si es true, se esta abriendo, si es false, se esta cerrando.
     * En newline se indica si al final del texto se ha de insertar una linea libre en el codigo html.
     * Se requiere el uso de un array llamado AttribArray para leer y escribir los atributos.
     */

    if ($opening) {
        $op = '';
    } else {
        $op = '/';
    }
    /* Si opening es true, significa que el tag se está abriendo, por lo que no
     * incluyo la barra. Si es false, significa que el tag se está cerrando e
     * incluyo la barra.
     */

    if ($newline) {
        $nl = PHP_EOL;
    } else {
        $nl = "";
    }
    /* Si newline es true, luego del tag se va a seguir escribiendo en la linea siguiente. */

    if ($opening) {
        if (!count($AttribArray) == 0) {
            foreach ($AttribArray as $attrib => $value) {
                $at = $at . $attrib . '="' . $value . '" ';
            }
        }
    }
    /* Si el array de atributos que pasé como parámetro no es nulo, genero la cadena de
     * atributos para el tag. */

    if ($finalclose) {
        $fc = '/';
    } else {
        $fc = '';
    }
    /* Para aquellos tags que se declaran y cierran en un solo paso, se lee finalclose
     * para saber si se agrega o no la barra al final.
     */

    // Creo el tag propiamente dicho, verificando primero si he de imprimirlo directamente o
    // devolverlo como texto.
    if ($astext) {
        return '<' . $op . $name . ' ' . $at . $fc . '>' . $nl;
    } else {
        echo '<' . $op . $name . ' ' . $at . $fc . '>' . $nl;
    }
}

function processHyperLink($link) {
    global $system;
    global $pagemap;
    //Nota: El link es relativo al archivo que se está ejecutando.
    if (is_numeric($link)) {
        // Si es un Integer, significa que pasé como dato un ID.
        if ($system->pageNavigationWithID) {
            // Si las páginas se cargan mediante un ID
            if (strpos($_SERVER['REQUEST_URI'], '?') === false) {
                // Si no hay ID fijado previamente.
                $link = $_SERVER['REQUEST_URI'] . '?id=' . $link;
            } else {
                // Si vengo de una página referenciada por una ID, saco esa parte para el nuevo link.
                $link = substr($_SERVER['REQUEST_URI'], 0, strpos($_SERVER['REQUEST_URI'], '?')) . '?id=' . $link;
            }
        } else {
            // Si cuando se carga la página, el ID se asocia al nombre del archivo.
            $link = $pagemap[$link];
            //CORRECCION PARA QUE FUNCIONE EN DIRECTORIO EXTERNO
            $link = '/agoraengine' . $link;
        }
    } else {
        // Si no es un integer, significa que recibi como dato una url ajena.
        // Verifico si tiene o no el http://, y si le falta, se lo pongo.
        if (strpos($link, 'http://') === false) {
            $link = 'http://' . $link;
        }
    }
    return $link;
}

function utf8tohtml($utf8) {
    /*
     * UTF8toHTML
     *
     * Esta función toma los caracteres en UTF8 y los convierte a su correspondiente
     * texto HTML en ascii, apto para navegadores.
     */
    // Link para copiar los caracteres: http://www.utexas.edu/learn/html/spchar.html
    $conversions = array(
        // Hacer la lista nueva aca abajo.
        // Puntuacion
        '–' => '&#8211;',
        '—' => '&#8212;',
        '¡' => '&#161;',
        '¿' => '&#191;',
        '"' => '&#34;',
        '“' => '&#8220;',
        '”' => '&#8221;',
        "'" => '&#39;',
        '‘' => '&#8216;',
        '’' => '&#8217;',
        '«' => '&#171;',
        '»' => '&#187;',
        // Simbolos
        '&' => '&#38;',
        '¢' => '&#162;',
        '©' => '&#169;',
        '÷' => '&#247;',
        '>' => '&#62;',
        '<' => '&#60;',
        'µ' => '&#181;',
        '·' => '&#183;',
        '¶' => '&#182;',
        '±' => '&#177;',
        '€' => '&#8364;',
        '£' => '&#163;',
        '®' => '&#174;',
        '§' => '&#167;',
        '™' => '&#153;',
        '¥' => '&#165;',
        // Diacriticas

//        '' => '&#;',
//        '' => '&#;',
//        '' => '&#;',
//        '' => '&#;',



        // Lista vieja, borrar cuando la otra esté completa.
        'á' => '&#225;',
        'í' => '&#237;',
        'é' => '&#233;',
        'ú' => '&#250;',
        'ñ' => '&#328;',
        'ó' => '&#243;',
    );
    // Falta traer el resto de los caracteres.
    $utf8 = utf8_encode($utf8);
    $utf8 = strtr($utf8, $conversions);
    return $utf8;
}

class OutputManager {

    private $content;

    public function addOutput($text, $newline=true) {
        if ($newline) {
            $this->content = $this->content . $text . PHP_EOL;
        } else {
            $this->content = $this->content . $text;
        }
    }

    public function export() {
        echo $this->content;
    }

    public function export_text() {
        return $this->content;
    }

    public function flush() {
        $this->content = '';
    }

}
?>
