<?php
/**
 * @author Kevin Dewald
 * @author Manuel Nicanor Gioffré
 */

/* Usando este código, si el config y los errorcodes no se encuentran, que sea el kernel
 * quien administre ese error.
*/

// <editor-fold defaultstate="collapsed" desc="Definicion de constantes.">
/** Definición de constantes para acortar el proceso de llamado a los distintos
 * módulos. */
define('DAT','DataHandler');
define('GUI','GUIHandler');
define('INT','International');
define('SEC','Security');
define('SES','SessionHandler');
define('DBS',"DataHandler/sql");
define('FRM',"DataHandler/forms");
define('STR','Standard'); // Se distingue este del STD de los formatos, por seguridad.
define('SCR','Scripts');
define('CFG','config');

/** Definición de Lenguajes soportados por el Kernel */
define('en', 'English');
define('es', 'Espanol');

/** Definición de constantes de formato de páginas. */
// Esta va a terminar volando despues de los cambios.
define('STD', 'Standard');
// </editor-fold>

// <editor-fold defaultstate="collapsed" desc="Inicializacion del Kernel">
if(!@include_once ('config/config.php')) {
    // Si no encontre el config.php, puede ser porque el sistema no se encuentra instalado.
    // Busco el directorio de los archivos de instalación.
    if(is_dir('AECore/_installation_files')) {
        // Si el directorio de instalacion existe, busco el archivo del instalador
        if(is_file('AECore/_installation_files/install.php')) {
            // Si el archivo de instalacion existe, redirijo el header allí:
            // Creo primero la variable location, con el path al root del engine.
            $location = $_SERVER['HTTP_HOST'].rtrim(dirname($_SERVER['PHP_SELF']), '/\\');
            header("Location: http://$location/AECore/_installation_files/install.php");
        }else {
            // Si no encuentro el archivo instalador, genero un panic de que la instalación esta corrupta.
            panic2('Kernel', 5, 'Installation directory corrupt.', true);
        }
    }else {
        // Si el directorio de instalación no existe, genero un panic.
        panic2('Kernel', 0, 'Config file could not be loaded.', true);
    }
}
if(!@include_once ('config/errorcodes.php')) {
    //Si no se encuentra los errorcodes, panic.
    panic2('Kernel', 1, 'Errorcodes file could not be loaded.', true);
}
if(!@include_once ('config/pagemap.php')) {
    //Si no se encuentra el pagemap, los links no van a funcionar, panic.
    panic2('Kernel', 2, 'Config file could not be loaded.', true);
}
if(!@include_once ('Manager/managerpagemap.php')) {
    //Si no se encuentra el pagemap, alguien saboteo el manager, panic.
    panic2('Kernel', 3, 'Manager pagemap file could not be loaded.', true);
}
if(!@include_once ('config/sql/tablesStorage.php')) {
    //Si no se encuentra los errorcodes, panic.
    panic2('Kernel', 4, 'Critical tables could not be loaded.', true);
}
function StartKernel($manager=false) {
    /*
     * StartKernel
     *
     * Esta función reemplaza a la vieja función InitiateKernel para preparar al sistema
     * con todos los datos que necesita para ejecutar una página, sin importar si
     * es del manager o del usuario.
     *
     * Los datos de entrada son los siguientes:
     * ID: El ID de la página a cargar. Si es null se intenta averiguar el ID mediante el
     *     nombre del archivo. En caso de que no se pueda averiguar que página es
     *     se redirigirá al index.php. Lo mismo ocurre si el ID de la página indicada no
     *     coincide con ninguna página del sistema.
     * Lang: Idioma para cargar la página. Si es null o no está en la lista de idiomas
     *       aceptados, se carga el idioma definido como estándar.
     * Manager: Si es true, se utilizarán las mascaras del manager correspondientes
     *          a la ID indicada. Por default es false.
     *
    */

    // Variables globales
    global $system;
    global $pagemap;
    global $managerpagemaps;
    global $primaryDatabase;
    global $DatabaseManager;
    // global $dbData;
    // PASO 1: Vincular e iniciar la base de datos primaria.


    // Vinculo al kernel el módulo de bases de datos.
    attachModule(DBS);
    // Declaro la variable DatabaseManager, que va a ser la que gestione las conecciones a bases de datos.
    $DatabaseManager = new database("primarydatabase");
    // Conecto DatabaseManager a la base de datos
    $DatabaseManager->connect();
    // Si la coneccion no funcionó, el databasemanager llama a la función panic.
    // Declaro la variable $method para propiedades extra del manejo de bases de datos.
    $method = new stdClass();
    $method->recursiveValues = true;

    
// PASO 2: Definir la página y el idioma que se va a utilizar.

    //Determino primero si estoy ejecutando una página del manager o una página del resto del sistema
    if($manager == true) {
        // Si estoy cargando una página del manager, necesito verificar la sesión para asegurarme que funciona bien

        // SISTEMA DE VALIDACION DE SESIONES PROVISORIO.
        // User: admin - Pass: 1234
        // User: owner - Pass: 1234

        session_start();
        if($_SESSION['validated']) {
            // Si ya se hizo login, se corroboran los datos.
            $user = $_SESSION['user'];
            $pass = $_SESSION['pass'];
            if ($pass != '1234' || ($user != 'admin' && $user != 'owner')) {
                // Si el password o el usuario son incorrectos, se destruyen todos los valores de la sesion
                unset ($_SESSION);
                // Se redirige el header a la pagina de login.
                header('Location: '.'http://'.$_SERVER['HTTP_HOST'].$system->relativePath.'/AECore/Manager/pages/loginpage.php?retry=true');
            }
        }else {
            // Si la sesion no esta validada, intento validarla.
            $user = $_POST['user'];
            $pass = $_POST['pass'];
            if ($pass != '1234' || ($user != 'admin' && $user != 'owner')) {
                // Si el password o el usuario son incorrectos, redirijo a la pagina de login
                // Verifico primero si se ingresaron datos o no
                if(isset($_POST['user']) || isset($_POST['pass'])) {
                    // Si se ingresaron datos, redirijo al retry
                    header('Location: '.'http://'.$_SERVER['HTTP_HOST'].$system->relativePath.'/AECore/Manager/pages/loginpage.php?retry=true');
                }else {
                    // Si no se ingresaron datos, redirijo al login comun.
                    header('Location: '.'http://'.$_SERVER['HTTP_HOST'].$system->relativePath.'/AECore/Manager/pages/loginpage.php');
                }
            } else {
                // Si llegue aca, es que el username y password estan bien.
                $_SESSION['user'] = $user;
                $_SESSION['pass'] = $pass;
                $_SESSION['validated'] = true;
                $_SESSION['type'] = $user;
            }
        }

        // Si estoy aca, es que ya terminó la validación de las sesiones.

        // Extraigo la ID de la pagina que quiero visitar.
        $ID = $_GET['id'];
        // Si la ID es nula, se le asigna por defecto el valor del MainMenu
        if($ID === null) {
            $ID = 0;
        }
        // Asigno el pagemap que tengo que usar.
        $validpagemap =& $managerpagemaps[$_SESSION['type']];
        // Verifico que la ID sea válida
        if(!key_exists($ID, $validpagemap)) {
            // Si la ID no es valida, voy al MainMenu
            $ID = 0;
        }
        // Fijo el path a la máscara.
        $path = $system->managermaskDir.$_SESSION['type'].'/'.$validpagemap[$ID];
    }else {
        $ID = $_GET['id'];
        // Si el ID es 1, redirijo la página hacia el manager.
        if($ID==1) {
            header('Location: '.'http://'.$_SERVER['HTTP_HOST'].$pagemap[1]);
        }
        // Verifico primero si tengo habilitada la navegación por ID.
        if($system->pageNavigationWithID) {
            // Como estoy navegando por ID, realizo verificaciones para confirmar el ID.
            if($ID==null) {
                /* Si tengo habilitada la navegación por ID e ID == null, intento averiguar
                 * el ID de la página que está llamando al Kernel y modifico el header para
                 * que cargue esa página. */

                // Declaro la variable auxiliar uri, que maneja la URL.
                $uri = $_SERVER['REQUEST_URI'];
                // Verifico si la URL tiene parámetros adicionales.
                if (strpos($uri,'?') == true) {
                    // Si tiene parámetros adicionales, se los saco.
                    $uri = substr($uri,0,strpos($uri,'?'));
                }
                // Cargo el modulo de funciones de busqueda.
                attachModule(STR, 'SearchFunctions');
                // Verifico si a la URL se le está indicando el archivo solicitado.
                if (EndsWith($uri, '/')) {
                    // Si la URL no tiene un nombre de archivo indicado, significa que voy al index.
                    $ID = 0;
                }else {
                    // Si hay un nombre de archivo indicado, busco su ID en el pagemap.
                    echo $uri;
                    // REVISAR: Si entro via: agoraengine/index.php, no encuentra el ID de la pagina.
                    $ID = array_search($uri, $pagemap);
                    // Verifico el resultado que obtuve.
                    if ($ID===false) {
                        // Si esta parte del código se ejecuta, el ID no se encuentra en el pagemap.
                        if($system->debugMode) {
                            // Si estoy en debugMode, genero un panic crítico.
                            panic2('Kernel', 16, 'This link was accessed: '.$_SERVER['REQUEST_URI'], true);
                        }else {
                            // Si no estoy en debugMode, el panic es invisible, y fijo el ID del index principal, o sea, el 0.
                            panic2('Kernel', 16, 'This link was accessed: '.$_SERVER['REQUEST_URI']);
                            $ID = 0;
                        }
                    }
                    // Como estoy navegando por ID, no debo aceptar URL's con archivos, por lo que modifico el Header.
                    header('Location: '.'http://'.$_SERVER['HTTP_HOST'].$system->relativePath.'/index.php?id='.$ID);
                }
            }else {
                /* Si esto se ejecuta, es que me indicaron la ID de la página. Antes de cargar
                 * el archivo correspondiente de la máscara, verifico que sea un número de ID
                 * válido. */
                if ($pagemap[$ID]=== null) {
                    // Si esta parte del código se ejecuta, el ID no se encuentra en el pagemap.
                    if($system->debugMode) {
                        // Si estoy en debugMode, genero un panic crítico.
                        panic2('Kernel', 16, 'This link was accessed: '.$_SERVER['REQUEST_URI'], true);
                    }else {
                        // Si no estoy en debugMode, el panic es invisible, y fijo el ID del index principal, o sea, el 0.
                        panic2('Kernel', 16, 'This link was accessed: '.$_SERVER['REQUEST_URI']);
                        $ID = 0;
                    }
                }
            }
        }else {
            /* Si no tengo habilitada la navegación por ID, necesito simplemente
             * averiguar el ID que corresponde al archivo, ignorando la petición de ID que se haga.
            */

            // Declaro la variable auxiliar uri, que maneja la URL.
            $uri = $_SERVER['REQUEST_URI'];
            // Verifico si la URL tiene parámetros adicionales.
            if (strpos($uri,'?') == true) {
                // Si tiene parámetros adicionales, se los saco.
                $uri = substr($uri,0,strpos($uri,'?'));
            }
            // Cargo el modulo de funciones de busqueda.
            attachModule(STR, 'SearchFunctions');
            // Verifico si a la URL se le está indicando el archivo solicitado.
            if (EndsWith($uri, '/')) {
                // Si la URL no tiene un nombre de archivo indicado, significa que voy al index.
                $ID = 0;
            }else {
                // Si hay un nombre de archivo indicado, busco su ID en el pagemap.
                $ID = array_search($uri, $pagemap);
                // Verifico el resultado que obtuve.
                if ($ID===false) {
                    // Si esta parte del código se ejecuta, el ID no se encuentra en el pagemap.
                    if($system->debugMode) {
                        // Si estoy en debugMode, genero un panic crítico.
                        panic2('Kernel', 16, 'This link was accessed: '.$_SERVER['REQUEST_URI'], true);
                    }else {
                        // Si no estoy en debugMode, el panic es invisible, y fijo el ID del index principal, o sea, el 0.
                        panic2('Kernel', 16, 'This link was accessed: '.$_SERVER['REQUEST_URI']);
                        $ID = 0;
                    }
                }
            }
        }
        // Tengo definido el número de ID de la máscara a extraer, voy a generar el path al archivo que la contiene.
        $path = $system->maskDir.'page'.$ID.'.msk';
    }
    // Extraigo de la base de datos la lista de lenguajes disponibles.

    //$languages = $DatabaseManager->retrieveDataById('engineLangs',array('short'), null, null,null, $method,'primarydatabase');
    $languages = $DatabaseManager->search("engineLangs", "short",null,null,null,null,null,null,"primarydatabase");
    // Extraccion de la lista de idiomas provisoria hasta la nueva version de retrieveDbData.

    foreach ($languages as $value) {
        $aux[]=$value['short'];
    }
    $languages = $aux;
    // Fin codigo provisorio.

    // Verifico primero si hay soporte para multiples idiomas
    if($system->multipleLanguageSupport==true) {
        // Como hay soporte para multiples idiomas, extraigo el valor de GET
        $lang = $_GET['lang'];
        // Si hay, verifico que idioma debo arrancar.
        if($lang==null) {
            // Si no se indica idioma, utilizo el estándar definido en config.php
            $GLOBALS['currentLANG'] = $languages[0];
        }else {
            // Si se indica un idioma, verifico primero que se encuentre dentro de los idiomas soportados
            $is_valid = array_search($lang, $languages);
            if ($is_valid === false) {
                // Si el idioma no está soportado, se carga el idioma estándar.
                $GLOBALS['currentLANG'] = $languages[0];
                // Genero un aviso que alguien pidió un idioma no soportado
                panic2('Kernel', 17, 'Unsupported language '.$lang.' has been requested.');
            }else {
                // Si el idioma está soportado, lo fijo como idioma de la página.
                $GLOBALS['currentLANG'] = $languages[$is_valid];
            }
        }
    }else {
        // Si el sistema no soporta multiples idiomas, se carga automáticamente el estándar.
        $GLOBALS['currentLANG'] = $languages[0];
    }

// PASO 3: Inicializar el renderer y generar la página.
    attachModule(GUI, 'renderer');
    Renderpage($path);
}

/*
 * HAY QUE AGREGAR UNA VARIABLE QUE DIGA QUE EL KERNELL ARRANCO CORRECTAMENTE
 * DE ESTE MODO SE EVITA QUE UN USUARIO TENGA ACCESO (LIMITADO O ILIMITADO)
 * A ARCHIVOS QUE ÚNICAMENTE DEBERIAN SER ACCESIBLES A TRAVES DE "includes"
 * INTERNOS DEL SISTEMA.
*/
// </editor-fold>


function attachModule($ModName,$ModClass = null) {
    /*
     * Función attachModule - Version 2
     *
     * Esta función generaliza el cargado de modulos, manteniendo la funcionalidad
     * de sus originales.
     * Para el caso del sistema de bases de datos, el sistema de cargado es especial,
     * ya que se reconoce el módulo de base de datos a cargar mediante la configuración
     * del sistema.
    */
    // Declaro las variables globales que voy a usar.
    global $system;
    // Verifico si estoy cargando el módulo de las bases de datos.
    if ($ModName == 'DataHandler/sql') {
        // El llamado a la base de datos es dinámico, dependiendo del databaseType indicado en $system.
        if(!@include_once($ModName.'/database.php')) {
            panic2('Kernel', 18, 'The file '.$system->databaseType.'Manager.php'.' inside the module '.$ModName.' could not be loaded.', true);
        }
        if(!@include_once($ModName.'/'.$system->databaseType.'Manager.php')) {
            panic2('Kernel', 18, 'The file '.$system->databaseType.'Manager.php'.' inside the module '.$ModName.' could not be loaded.', true);
        }
        if(!@include_once($ModName.'/sqlInterface.php')) {
            panic2('Kernel', 18, 'The file sqlInterface.php'.' inside the module '.$ModName.' could not be loaded.', true);
        }
    }
    else {
        // Si el módulo que estoy cargando no es el de bases de datos, hago uno genérico.
        if(!@include_once($ModName.'/'.$ModClass.'.php')) {
            // Si no se pudo cargar el módulo, panic.
            panic2('Kernel', 18, 'The file '.$ModClass.' inside the module '.$ModName.' could not be loaded.', true);
        }
    }
}
function alert($module,$resource,$notify) {

    //IMPORTANTE: Función deprecada, usar panic2.

    /** Esta función genera una alerta hacia el usuario, sin necesidad de interrumpir la
     * ejecución del código. Sería necesario alguna función para llevar registro de estos
     * avisos. En $module se indica el módulo que está generando el error, en
     * $resource el error en si, y en $notify se incorpora un boolean para
     * indicar si se debe informar al usuario del aviso. */
    global $system;
    switch ($module) {
        case 'renderer':
            if($system->debugMode) {
                //nota: Sistema de log de errores
                echo("A warning has been detected.<br/>Warning description: ".$resource);
            }else {
                if ($notify) {
                    echo("A warning has been detected and reported, but will not affect your navigation experience.<br/>");
                    // Aca viene la función que genere un log del error.
                }else {
                    // Aca viene la funcion que genere un log del error.
                }
            }
            break;

    }
}
function panic($module,$resource,$errorcode = NULL) {

    //IMPORTANTE: Función deprecada, usar panic2.

    /* Agregué aca arriba el parámetro errorCode. La idea es usarlo para cuando una
 * página tenga un error fuera del debugmode y lo vea el usuario. Al reportar el
 * errorCode, facilita muchísimo la tarea de rastrear un error, que a veces puede
 * no producirse en modo debug. El concepto es que cada llamado de error tenga un
 * número asociado, y que ese número corresponda a un punto único en todo el kernel.
 * Cuando el código se vuelve grande, tener reportes de errores para puntos
 * específicos en el código es la clave para arreglar bugs.
    */



    global $system;
    /* si el sistema esta en modo debug va a mostrar los errores cuando se
     * generen. Si el sistema esta en modo standard, no es conveniente que
     * el usuario vea los errores.
    */
    switch ($module) {
        case is_array($module):
            if($system->debugMode) {
                if($module[0] == "kernelLoadModuleFailure") {
                    die("A critical error has occurred and the system can not continue to load: ".$resource);
                }
                if($module[0] == "criticalModuleMissing") {
                    die("A critical error has occurred and the system can not continue to load: ".$resource);
                }
                if($module[0] == "PageMissing") {
                    if($system->debugMode) {
                        die("A critical error has occurred and the system can not continue to load: ".$resource);
                    }else {
                        if (isset($errorcode)) {
                            die("This page cannot be loaded. <br/> Please report the following error code to the system administrator: CODE".$errorcode);
                        }else {
                            die('This page cannot be loaded. Please contact your system administrator to inform a broken link.');
                        }
                    }
                }

            }
            break;
        case "dataHandler/sql":
            if($system->databaseErrorCreatesPanic) {
                if($system->debugMode) {
                    //nota: Sistema de log de errores
                    die("A critical error has occurred and the system can not continue to load: ".$resource);
                }else {
                    die("A critical error has occurred and the system can not continue to load. Please contact the administrator");
                }
            }else {
                if($system->debugMode) {
                    //nota: Sistema de log de errores
                    echo("A critical error has happened, this will not stop the system. Error description: ".$resource);
                }else {
                    die("An error ocurred, you can not log into the database but the system will continue to load");
                }
            }
            break;
        case 'renderer':
            if($system->debugMode) {
                //nota: Sistema de log de errores
                echo("A critical error has happened.<br/>Error description: ".$resource);
            }else {
                if (isset($errorcode)) {
                    die("This page cannot be loaded. <br/> Please report the following error code to the system administrator: CODE".$errorcode);
                }else {
                    die('This page cannot be loaded. Please contact your system administrator');
                }
            }
            break;
        case 'language':
            if($system->debugMode) {
                //nota: Sistema de log de errores
                echo("A critical error has happened.<br/>Error description: ".$resource);
            }else {
                if (isset($errorcode)) {
                    die("This page cannot be loaded. <br/> Please report the following error code to the system administrator: CODE".$errorcode);
                }else {
                    die('This page cannot be loaded. Please contact your system administrator');
                }
            }
            break;
    }

}
function panic2($module,$errorcode,$errorinfo,$critical=false,$visible=false) {
    /*
 * Función Panic - Version 2
 *
 * Esta función de pánico es mas flexible que se predecesora, comprimiendo en
 * una función también a la vieja función alert.
 *
 * $module: Se indica el módulo que está generando el panic.
 * $errorcode: Se indica el código de error generado, de acuerdo a la tabla errorcodes.php
 * $errorinfo: Información adicional del error.
 * $critical: Si el llamado a panic tiene que detener la ejecución de la pagina.
 * $visible: Si no se detiene la ejecución del programa, si se debe o no generar una advertencia visible.
 *
    */
    global $system;
    // Reporto el error con la función reporterror
    reporterror($module, $errorcode, $errorinfo, $critical);
    // Verifico primero si la página esta operando en debugMode
    if($system->debugMode) {
        /* Si la página está operando en debugMode, todos los errores deben
         * reportarse al usuario a fin de que pueda tomar medidas pertinentes.*/
        if ($critical) {
            // Si el error es crítico, detengo la ejecución y reporto el error.
            die ('Module '.$module.' has issued the following critical error code '.$errorcode.'.<br/>'.'The following information was provided: <br/>'.$errorinfo.'.<br/>');
        }else {
            echo ('Module '.$module.' has issued the following non critical error code '.$errorcode.'.<br/>'.'The following information was provided: <br/>'.$errorinfo.'.<br/>');
        }
    }else {
        if ($critical) {
            // Si el error es crítico, detengo la ejecución y reporto el error.
            die ('The critical error code '.$errorcode.' has been detected.'.'<br/>');
        }else {
            // Si el error no es crítico, verifico si debe mostrarse o no.
            if ($visible) {
                // Si el error es visible, hago un echo con el mensaje de error.
                echo 'The non critical error code '.$errorcode.' has been detected.'.'.<br/>';
            }
        }
    }
}
function reporterror($module,$errorcode,$errorinfo,$critical) {
    /*
     * Esta función escribe dentro del archivo de errorlog la información acerca
     * del error que generó el panic.
     * Leo primero el archivo para recuperar la información que ya existe. Si no
     * existe el archivo, lo creo.
    */
    global $system;
    // Incluyo el modulo de seguridad
    attachModule(SEC, 'SecurityFunctions');
    // Extraigo los contenidos del archivo.
    $errorlog = SafeUnSerialize($system->errorFile);
    // Si el archivo no existe, lo creo.
    if($errorlog === false) {
        // Instancio una nueva clase de errorlog.
        $errorlog = new ErrorLog();
        // Agrego el nuevo error al log.
        $errorlog->addError($module,$errorcode,$errorinfo,$critical);
        // Serializo de manera segura la clase errorlog.
        SafeSerialize($errorlog, $system->errorFile);
    }else {
        // Como el archivo existe, instancio la clase errorlog.
        $errorlog = unserialize($errorlog);
        // Agrego el nuevo error al log.
        $errorlog->addError($module,$errorcode,$errorinfo,$critical);
        // Serializo de manera segura la clase errorlog.
        SafeSerialize($errorlog, $system->errorFile);
    }
}

/** El sistema de alias administra los resource links para tener un acceso rapido
 sin necesidad de reconectar la base de datos
 **/
$dbRL = Array();
// Establece un nuevo DB Alias
function newDBRL($alias,$rl) {
    global $dbRL;
    $dbRL[$alias] = $rl;
}
// Devuelve el DB Alias si es que existe
function getDBRByAlias($alias) {
    global $dbRL;
    if(isset($dbRL[$alias])) {
        return $dbRL[$alias];
    }else {
        return false;
    }
}
/*----------------------------------------------------------------------------*/
// Declaración de clases internas.
class ErrorLog {
    /*
     * Clase ErrorLog
     *
     * En esta clase se almacenan los reportes de errores obtenidos, a fin de llevar
     * registro de ellos y poder hacer las correcciones correspondientes al sistema.
     *
     * Los datos almacenados son:
     * - Número unico del error
     * - Hora en la que se produjo el error. (Formato: 'Y-m-d H:i:s \G\M\TP')
     * - Modulo donde se produjo el error.
     * - Código del error.
     * - Información adicional del error.
     * - Si el error produjo una detención crítica.
    */

    private $errorarray = array();
    private $errornumber = 1;

    public function addError($module,$errorcode,$errorinfo,$critical) {
        /*
         * Esta función crea un array con toda la información de un error y lo
         * agrega al array general de errores.
         * Se incrementa también el contador del número único del error.
        */
        $this->errorarray[] = array($this->errornumber,date('Y-m-d H:i:s \G\M\TP'),$module,$errorcode,$errorinfo,$critical);
        $this->errornumber++;
    }

    public function archiveLog() {
        /*
         * Esta función elimina el registro de todos los errores, manteniendo
         * el contador. Los registros eliminados se almacenan temporariamente en
         * un arreglo, que es devuelto a quien llama a la función.
        */
        // Creo la variable de retorno.
        $retarray = $this->errorarray;
        // Una vez archivados los valores, pongo a cero el arreglo.
        $this->errorarray = array();
        return $retarray;
    }

    public function archiveOlder($date) {
        /*
         * Esta función recibe como fecha cualquier formato que pueda ser parseado
         * por la función de php strtotime().
         * Quita del array de errores todas aquellas entradas mas viejas que la fecha
         * especificada y las devuelve para ser archivadas.
        */
        // Creo la variable de retorno.
        $retarray = array();
        // Convierto date en un formato de fecha con el que puedo trabajar y comparar.
        $date = strtotime($date, strtotime('now'));
        // Verifico que date sea válida, consultando el resultado de la funcion strtotime.
        if ($date !== false) {
            // Siendo date válida, prosigo eliminando las entradas viejas.
            foreach ($this->errorarray as $key => $value) {
                // El value obtenido es el arreglo con toda la información del error.
                // El index 1 de value es la fecha del error.
                // Archivo el error solamente si su fecha es anterior a la indicada.
                if (strtotime($value[1], strtotime('now')) < $date) {
                    // Guardo la info en el arreglo a ser devuelto.
                    $retarray[] = $value;
                    // Lo quito del arreglo.
                    unset ($this->errorarray[$key]);
                }
            }
            // Una vez procesados todos los elementos del arreglo, lo reordeno para acomodar los índices.
            $this->errorarray = array_values($this->errorarray);
            // Devuelvo el arreglo con los valores a ser archivados.
            return $retarray;

        }
    }
    public function archiveID($ID) {
        /*
         * Esta función archiva solamente el error cuya ID coincide con la ID especificada.
         * Se devuelve la información del error a ser archivado.
        */
        foreach ($this->errorarray as $key => $value) {
            // El value obtenido es el arreglo con toda la información del error.
            // El index 0 de value es el ID del error.
            // Archivo el error solamente si ID coincide.
            if ($value[1] == $ID) {
                // Copio el error a retarray para ser devuelto
                $retarray = $value;
                // Lo quito del arreglo.
                unset ($this->errorarray[$key]);
                // Termino la ejecución del foreach.
                break;
            }
        }
        // Una vez procesados todos los elementos del arreglo, lo reordeno para acomodar los índices.
        $this->errorarray = array_values($this->errorarray);
        // Devuelvo el error para que sea archivado.
        return $retarray;
    }
}

function getDBM() {
    global $DatabaseManager;
    return $DatabaseManager;
}
/*----------------------------------------------------------------------------*/
// Declaración de variables globales
global $system;
global $currentLANG;
global $openPage;
global $pagemap;
global $managerpagemaps;
global $primaryDatabase;
global $DatabaseManager;
global $dbData;
/*----------------------------------------------------------------------------*/

?>
